@@ -17,6 +17,7 @@
 #include <string.h>
 #include <ctype.h>
 #include "bind.h"
+#include "misc.h"
 
 #include "environ.h"
 #include "statement.h"
@@ -73,7 +74,8 @@ PGAPI_BindParameter(
 	/* store the given info */
 	apdopts->parameters[ipar].buflen = cbValueMax;
 	apdopts->parameters[ipar].buffer = rgbValue;
-	apdopts->parameters[ipar].used = pcbValue;
+	apdopts->parameters[ipar].used =
+	apdopts->parameters[ipar].indicator = pcbValue;
 	apdopts->parameters[ipar].CType = fCType;
 	ipdopts->parameters[ipar].SQLType = fSqlType;
 	ipdopts->parameters[ipar].paramType = fParamType;
@@ -187,7 +189,8 @@ PGAPI_BindCol(
 			if (bookmark)
 			{
 				bookmark->buffer = NULL;
-				bookmark->used = NULL;
+				bookmark->used =
+				bookmark->indicator = NULL;
 			}
 		}
 		else
@@ -209,7 +212,8 @@ inolog("Bind column 0 is type %d not of type SQL_C_BOOKMARK", fCType);
 
 			bookmark = ARD_AllocBookmark(opts);
 			bookmark->buffer = rgbValue;
-			bookmark->used = pcbValue;
+			bookmark->used =
+			bookmark->indicator = pcbValue;
 			bookmark->buflen = cbValueMax;
 			bookmark->returntype = fCType;
 		}
@@ -246,7 +250,8 @@ inolog("Bind column 0 is type %d not of type SQL_C_BOOKMARK", fCType);
 		/* we have to unbind the column */
 		opts->bindings[icol].buflen = 0;
 		opts->bindings[icol].buffer = NULL;
-		opts->bindings[icol].used = NULL;
+		opts->bindings[icol].used =
+		opts->bindings[icol].indicator = NULL;
 		opts->bindings[icol].returntype = SQL_C_CHAR;
 		opts->bindings[icol].precision = 0;
 		opts->bindings[icol].scale = 0;
@@ -261,7 +266,8 @@ inolog("Bind column 0 is type %d not of type SQL_C_BOOKMARK", fCType);
 		/* ok, bind that column */
 		opts->bindings[icol].buflen = cbValueMax;
 		opts->bindings[icol].buffer = rgbValue;
-		opts->bindings[icol].used = pcbValue;
+		opts->bindings[icol].used =
+		opts->bindings[icol].indicator = pcbValue;
 		opts->bindings[icol].returntype = fCType;
 #if (ODBCVER >= 0x0300)
 		if (SQL_C_NUMERIC == fCType)
@@ -364,6 +370,12 @@ ipdopts->parameters[ipar].PGType);
 			*pfSqlType = ipdopts->parameters[ipar].SQLType;
 		else if (ipdopts->parameters[ipar].PGType)
 			*pfSqlType = pgtype_to_concise_type(stmt, ipdopts->parameters[ipar].PGType, PG_STATIC);
+		else
+		{
+			ret = SQL_ERROR;
+			SC_set_error(stmt, STMT_EXEC_ERROR, "Unfortunatley couldn't get this paramater's info", func);
+			goto cleanup;
+		}
 	}
 
 	if (pcbParamDef)
@@ -462,101 +474,15 @@ inolog("num_params=%d,%d\n", stmt->num_params, stmt->proc_return);
 	}
 	else
 	{
-		const	char *sptr, *tag = NULL;
-		ConnectionClass	*conn = SC_get_conn(stmt);
-		size_t	taglen = 0;
-		char	tchar, bchar, escape_in_literal = '\0';
-		char	in_literal = FALSE, in_identifier = FALSE,
-			in_dollar_quote = FALSE, in_escape = FALSE,
-			multi = FALSE, del_found = FALSE;
-		encoded_str	encstr;
+		char	multi = FALSE, proc_return = 0;
 
 		stmt->proc_return = 0;
-		make_encoded_str(&encstr, conn, stmt->statement);
-		for (sptr = stmt->statement, bchar = '\0'; *sptr; sptr++)
-		{
-			tchar = encoded_nextchar(&encstr);
-			if (ENCODE_STATUS(encstr) != 0) /* multibyte char */
-			{
-				if ((UCHAR) tchar >= 0x80)
-					bchar = tchar;
-				continue;
-			}
-			if (!multi && del_found)
-			{
-				if (!isspace(tchar))
-					multi = TRUE;
-			}
-			if (in_dollar_quote)
-			{
-				if (tchar == dollar_quote)
-				{
-					if (strncmp(sptr, tag, taglen) == 0)
-					{
-						in_dollar_quote = FALSE;
-						tag = NULL;
-						sptr += taglen;
-						sptr--;
-						encoded_position_shift(&encstr, taglen - 1);
-					}
-				}
-			}
-			else if (in_literal)
-			{
-				if (in_escape)
-					in_escape = FALSE;
-				else if (tchar == escape_in_literal)
-					in_escape = TRUE;
-				else if (tchar == literal_quote)
-					in_literal = FALSE;
-			}
-			else if (in_identifier)
-			{
-				if (tchar == identifier_quote)
-					in_identifier = FALSE;
-			}
-			else
-			{
-				if (tchar == '?')
-				{
-					if (0 == *pcpar && bchar == '{')
-						stmt->proc_return = 1;
-					(*pcpar)++;
-				}
-				else if (tchar == ';')
-					del_found = TRUE;
-				else if (tchar == dollar_quote)
-				{
-					char	*dollar_next;
-
-					in_dollar_quote = TRUE;
-					tag = sptr;
-					taglen = 0; 
-					if (dollar_next = strchr(sptr + 1, dollar_quote), NULL != dollar_next)
-					{
-						taglen = dollar_next - sptr + 1;
-						sptr = dollar_next;
-						encoded_position_shift(&encstr, taglen - 1);
-					}
-				}
-				else if (tchar == literal_quote)
-				{
-					in_literal = TRUE;
-					escape_in_literal = CC_get_escape(conn);
-					if (!escape_in_literal)
-					{
-						if (LITERAL_EXT == sptr[-1])
-							escape_in_literal = ESCAPE_IN_LITERAL;
-					}
-				}
-				else if (tchar == identifier_quote)
-					in_identifier = TRUE;
-				if (!isspace(tchar))
-					bchar = tchar;
-			}
-		}
+		SC_scanQueryAndCountParams(stmt->statement, SC_get_conn(stmt), NULL, pcpar, &multi, &proc_return);
 		stmt->num_params = *pcpar;
+		stmt->proc_return = proc_return;
 		stmt->multi_statement = multi;
+		if (multi)
+			SC_no_parse_tricky(stmt);
 	}
 inolog("num_params=%d,%d\n", stmt->num_params, stmt->proc_return);
 	return SQL_SUCCESS;
@@ -574,13 +500,14 @@ create_empty_bindings(int num_columns)
 
 	new_bindings = (BindInfoClass *) malloc(num_columns * sizeof(BindInfoClass));
 	if (!new_bindings)
-		return 0;
+		return NULL;
 
 	for (i = 0; i < num_columns; i++)
 	{
 		new_bindings[i].buflen = 0;
 		new_bindings[i].buffer = NULL;
-		new_bindings[i].used = NULL;
+		new_bindings[i].used =
+		new_bindings[i].indicator = NULL;
 	}
 
 	return new_bindings;
@@ -592,7 +519,7 @@ extend_parameter_bindings(APDFields *self, int num_params)
 	CSTR func = "extend_parameter_bindings";
 	ParameterInfoClass *new_bindings;
 
-	mylog("%s: entering ... self=%p, parameters_allocated=%d, num_params=%d\n", func, self, self->allocated, num_params);
+	mylog("%s: entering ... self=%p, parameters_allocated=%d, num_params=%d,%p\n", func, self, self->allocated, num_params, self->parameters);
 
 	/*
 	 * if we have too few, allocate room for more, and copy the old
@@ -609,14 +536,13 @@ extend_parameter_bindings(APDFields *self, int num_params)
 			self->allocated = 0;
 			return;
 		}
-		memset(&new_bindings[self->allocated], 0,
-			sizeof(ParameterInfoClass) * (num_params - self->allocated));
+		memset(&new_bindings[self->allocated], 0, sizeof(ParameterInfoClass) * (num_params - self->allocated));
 
 		self->parameters = new_bindings;
 		self->allocated = num_params;
 	}
 
-	mylog("exit extend_parameter_bindings\n");
+	mylog("exit %s=%p\n", func, self->parameters);
 }
 
 void
@@ -649,7 +575,7 @@ extend_iparameter_bindings(IPDFields *self, int num_params)
 		self->allocated = num_params;
 	}
 
-	mylog("exit extend_iparameter_bindings\n");
+	mylog("exit %s=%p\n", func, self->parameters);
 }
 
 void
@@ -664,8 +590,9 @@ reset_a_parameter_binding(APDFields *self, int ipar)
 
 	ipar--;
 	self->parameters[ipar].buflen = 0;
-	self->parameters[ipar].buffer = 0;
-	self->parameters[ipar].used = 0;
+	self->parameters[ipar].buffer = NULL;
+	self->parameters[ipar].used =
+	self->parameters[ipar].indicator = NULL;
 	self->parameters[ipar].CType = 0;
 	self->parameters[ipar].data_at_exec = FALSE;
 	self->parameters[ipar].precision = 0;
@@ -743,7 +670,8 @@ CountParameters(const StatementClass *self, Int2 *inputCount, Int2 *ioCount, Int
 void
 APD_free_params(APDFields *apdopts, char option)
 {
-	mylog("APD_free_params:  ENTER, self=%d\n", apdopts);
+	CSTR	func = "APD_free_params";
+	mylog("%s:  ENTER, self=%p\n", func, apdopts);
 
 	if (!apdopts->parameters)
 		return;
@@ -755,15 +683,16 @@ APD_free_params(APDFields *apdopts, char option)
 		apdopts->allocated = 0;
 	}
 
-	mylog("APD_free_params:  EXIT\n");
+	mylog("%s:  EXIT\n", func);
 }
 
 void
 PDATA_free_params(PutDataInfo *pdata, char option)
 {
+	CSTR	func = "PDATA_free_params";
 	int			i;
 
-	mylog("PDATA_free_params:  ENTER, self=%p\n", pdata);
+	mylog("%s:  ENTER, self=%p\n", func, pdata);
 
 	if (!pdata->pdata)
 		return;
@@ -789,7 +718,7 @@ PDATA_free_params(PutDataInfo *pdata, char option)
 		pdata->allocated = 0;
 	}
 
-	mylog("PDATA_free_params:  EXIT\n");
+	mylog("%s:  EXIT\n", func);
 }
 
 /*
@@ -798,7 +727,9 @@ PDATA_free_params(PutDataInfo *pdata, char option)
 void
 IPD_free_params(IPDFields *ipdopts, char option)
 {
-	mylog("IPD_free_params:  ENTER, self=%p\n", ipdopts);
+	CSTR	func = "IPD_free_params";
+
+	mylog("%s:  ENTER, self=%p\n", func, ipdopts);
 
 	if (!ipdopts->parameters)
 		return;
@@ -809,7 +740,7 @@ IPD_free_params(IPDFields *ipdopts, char option)
 		ipdopts->allocated = 0;
 	}
 
-	mylog("IPD_free_params:  EXIT\n");
+	mylog("%s:  EXIT\n", func);
 }
 
 void
@@ -863,7 +794,7 @@ extend_column_bindings(ARDFields *self, int num_columns)
 	/* SQLExecDirect(...)  # returns 5 cols */
 	/* SQLExecDirect(...)  # returns 10 cols  (now OK) */
 
-	mylog("exit extend_column_bindings\n");
+	mylog("exit %s=%p\n", func, self->bindings);
 }
 
 void
@@ -883,7 +814,8 @@ reset_a_column_binding(ARDFields *self, int icol)
 		if (bookmark = self->bookmark, bookmark != NULL)
 		{
 			bookmark->buffer = NULL;
-			bookmark->used = NULL;
+			bookmark->used =
+			bookmark->indicator = NULL;
 		}
 	}
 	else
@@ -893,7 +825,8 @@ reset_a_column_binding(ARDFields *self, int icol)
 		/* we have to unbind the column */
 		self->bindings[icol].buflen = 0;
 		self->bindings[icol].buffer = NULL;
-		self->bindings[icol].used = NULL;
+		self->bindings[icol].used =
+		self->bindings[icol].indicator = NULL;
 		self->bindings[icol].returntype = SQL_C_CHAR;
 	}
 }
@@ -1022,7 +955,7 @@ extend_getdata_info(GetDataInfo *self, int num_columns, BOOL shrink)
 	 * about it by unbinding those columns.
 	 */
 
-	mylog("exit extend_gdata_info\n");
+	mylog("exit extend_gdata_info=%p\n", self->gdata);
 }
 void	reset_a_getdata_info(GetDataInfo *gdata_info, int icol)
 {
@@ -1092,7 +1025,7 @@ extend_putdata_info(PutDataInfo *self, int num_params, BOOL shrink)
 		}
 	}
 
-	mylog("exit extend_putdata_info\n");
+	mylog("exit %s=%p\n", func, self->pdata);
 }
 void	reset_a_putdata_info(PutDataInfo *pdata_info, int ipar)
 {
