@@ -54,6 +54,8 @@ enum
 #define CONNECTION_NEED_PASSWORD				112
 #define CONNECTION_COMMUNICATION_ERROR				113
 
+#define CONN_TRUNCATED							(-2)
+#define CONN_OPTION_VALUE_CHANGED					(-1)
 /*	These errors correspond to specific SQL states */
 #define CONN_INIREAD_ERROR						201
 #define CONN_OPENDB_ERROR						202 
@@ -70,10 +72,8 @@ enum
 #define CONN_AUTH_TYPE_UNSUPPORTED					211
 #define CONN_UNABLE_TO_LOAD_DLL						212
 
-#define CONN_OPTION_VALUE_CHANGED					213
 #define CONN_VALUE_OUT_OF_RANGE						214
 
-#define CONN_TRUNCATED							215
 #define CONN_OPTION_NOT_FOR_THE_DRIVER					216
 #define CONN_EXEC_ERROR							217
 
@@ -170,11 +170,13 @@ do { \
 #define DELETE_CONNLOCK(x)	pthread_mutex_destroy(&((x)->slock))
 #else
 #define INIT_CONN_CS(x)	
-#define INIT_CONN_LOCK(x)	
+#define INIT_CONNLOCK(x)	
 #define TRY_ENTER_CONN_CS(x)
 #define ENTER_CONN_CS(x)
-#define ENTER_INNER_CONN_CS(x, entered) (0)
+#define CONNLOCK_ACQUIRE(x)
+#define ENTER_INNER_CONN_CS(x, entered)
 #define LEAVE_CONN_CS(x)
+#define CONNLOCK_RELEASE(x)
 #define DELETE_CONN_CS(x)
 #define DELETE_CONNLOCK(x)
 #endif /* WIN_MULTITHREAD_SUPPORT */
@@ -468,7 +470,7 @@ struct ConnectionClass_
 #define CC_get_username(x)			(x->connInfo.username)
 #define CC_is_onlyread(x)			(x->connInfo.onlyread[0] == '1')
 #define CC_get_escape(x)			(x->escape_in_literal)
-#define CC_fake_mss(x)		(0 != (x)->ms_jet && 0 < (x)->connInfo.fake_mss)
+#define CC_fake_mss(x)	(/* 0 != (x)->ms_jet && */ 0 < (x)->connInfo.fake_mss)
 #define CC_default_is_c(x)	(CC_is_in_ansi_app(x) || x->ms_jet /* not only */ || TRUE /* but for any other ? */)
 /*	for CC_DSN_info */
 #define CONN_DONT_OVERWRITE		0
@@ -541,13 +543,6 @@ enum {
 #define	NO_TRANS		1L
 #define	CONN_DEAD		(1L << 1) /* connection is no longer valid */
 
-#ifdef	WIN32
-#ifdef	_HANDLE_ENLIST_IN_DTC_
-RETCODE	EnlistInDtc(ConnectionClass *conn, void *pTra, int method);
-RETCODE	DtcOnDisconnect(ConnectionClass *);
-#endif /* _HANDLE_ENLIST_IN_DTC_ */
-#endif /* WIN32 */
-
 #ifdef	__cplusplus
 }
 #endif
