@@ -27,8 +27,8 @@
 
 extern GLOBAL_VALUES globals;
 
-static void encode(const char *in, char *out);
-static void decode(const char *in, char *out);
+static void encode(const UCHAR *in, UCHAR *out);
+static void decode(const UCHAR *in, UCHAR *out);
 
 UInt4	getExtraOptions(const ConnInfo *ci)
 {
@@ -412,9 +412,12 @@ unfoldCXAttribute(ConnInfo *ci, const char *value)
 	ci->use_server_side_prepare = (char)((flag & BIT_USESERVERSIDEPREPARE) != 0);
 	ci->lower_case_identifier = (char)((flag & BIT_LOWERCASEIDENTIFIER) != 0);
 }
-void
+BOOL
 copyAttributes(ConnInfo *ci, const char *attribute, const char *value)
 {
+	CSTR	func = "copyAttributes";
+	BOOL	found = TRUE;
+
 	if (stricmp(attribute, "DSN") == 0)
 		strcpy(ci->dsn, value);
 
@@ -536,13 +539,20 @@ copyAttributes(ConnInfo *ci, const char *attribute, const char *value)
 		}
 		mylog("force_abbrev=%d bde=%d cvt_null_date=%x\n", ci->force_abbrev_connstr, ci->bde_environment, ci->cvt_null_date_string);
 	}
+	else
+		found = FALSE;
 
-	mylog("copyAttributes: DSN='%s',server='%s',dbase='%s',user='%s',passwd='%s',port='%s',onlyread='%s',protocol='%s',conn_settings='%s',disallow_premature=%d)\n", ci->dsn, ci->server, ci->database, ci->username, ci->password ? "xxxxx" : "", ci->port, ci->onlyread, ci->protocol, ci->conn_settings, ci->disallow_premature);
+	mylog("%s: DSN='%s',server='%s',dbase='%s',user='%s',passwd='%s',port='%s',onlyread='%s',protocol='%s',conn_settings='%s',disallow_premature=%d)\n", func, ci->dsn, ci->server, ci->database, ci->username, ci->password ? "xxxxx" : "", ci->port, ci->onlyread, ci->protocol, ci->conn_settings, ci->disallow_premature);
+
+	return found;
 }
 
-void
+BOOL
 copyCommonAttributes(ConnInfo *ci, const char *attribute, const char *value)
 {
+	CSTR	func = "copyCommonAttributes";
+	BOOL	found = TRUE;
+
 	if (stricmp(attribute, INI_FETCH) == 0 || stricmp(attribute, ABBR_FETCH) == 0)
 		ci->drivers.fetch_max = atoi(value);
 	else if (stricmp(attribute, INI_SOCKET) == 0 || stricmp(attribute, ABBR_SOCKET) == 0)
@@ -583,7 +593,10 @@ copyCommonAttributes(ConnInfo *ci, const char *attribute, const char *value)
 		ci->drivers.bools_as_char = atoi(value);
 	else if (stricmp(attribute, INI_EXTRASYSTABLEPREFIXES) == 0 || stricmp(attribute, ABBR_EXTRASYSTABLEPREFIXES) == 0)
 		strcpy(ci->drivers.extra_systable_prefixes, value);
-	mylog("CopyCommonAttributes: A7=%d;A8=%d;A9=%d;B0=%d;B1=%d;B2=%d;B3=%d;B4=%d;B5=%d;B6=%d;B7=%d;B8=%d;B9=%d;C0=%d;C1=%d;C2=%s",
+	else
+		found = FALSE;
+
+	mylog("%s: A7=%d;A8=%d;A9=%d;B0=%d;B1=%d;B2=%d;B3=%d;B4=%d;B5=%d;B6=%d;B7=%d;B8=%d;B9=%d;C0=%d;C1=%d;C2=%s", func,
 		  ci->drivers.fetch_max,
 		  ci->drivers.socket_buffersize,
 		  ci->drivers.unknown_sizes,
@@ -600,6 +613,8 @@ copyCommonAttributes(ConnInfo *ci, const char *attribute, const char *value)
 		  ci->drivers.parse,
 		  ci->drivers.cancel_as_freestmt,
 		  ci->drivers.extra_systable_prefixes);
+
+	return found;
 }
 
 
@@ -858,49 +873,63 @@ getDSNinfo(ConnInfo *ci, char overwrite)
 		 ci->fake_oid_index,
 		 ci->show_system_tables);
 
-	check_client_encoding(ci->conn_settings);
-	qlog("          conn_settings='%s',conn_encoding='%s'\n",
-		 ci->conn_settings,
-		 check_client_encoding(ci->conn_settings));
-	qlog("          translation_dll='%s',translation_option='%s'\n",
-		 ci->translation_dll,
-		 ci->translation_option);
+	if (get_qlog())
+	{
+		UCHAR	*enc = check_client_encoding(ci->conn_settings);
+
+		qlog("          conn_settings='%s',conn_encoding='%s'\n",
+		 	ci->conn_settings, enc ? enc : "(null)");
+		if (enc)
+			free(enc);
+		qlog("          translation_dll='%s',translation_option='%s'\n",
+		 	ci->translation_dll,
+		 	ci->translation_option);
+	}
 }
 /*
  *	This function writes any global parameters (that can be manipulated)
  *	to the ODBCINST.INI portion of the registry
  */
-void
+int
 writeDriverCommoninfo(const char *fileName, const char *sectionName,
 			 const GLOBAL_VALUES *comval)
 {
 	char		tmp[128];
+	int		errc = 0;
 
 	if (ODBCINST_INI == fileName && NULL == sectionName)
 		sectionName = DBMS_NAME;
  
-	sprintf(tmp, "%d", comval->fetch_max);
-	SQLWritePrivateProfileString(sectionName,
-								 INI_FETCH, tmp, fileName);
-
 	sprintf(tmp, "%d", comval->commlog);
-	SQLWritePrivateProfileString(sectionName,
-								 INI_COMMLOG, tmp, fileName);
+	if (!SQLWritePrivateProfileString(sectionName, INI_COMMLOG, tmp, fileName))
+		errc--;
 
 	sprintf(tmp, "%d", comval->debug);
-	SQLWritePrivateProfileString(sectionName,
-								 INI_DEBUG, tmp, fileName);
+	if (!SQLWritePrivateProfileString(sectionName, INI_DEBUG, tmp, fileName))
+		errc--;
+
+	sprintf(tmp, "%d", comval->fetch_max);
+	if (!SQLWritePrivateProfileString(sectionName, INI_FETCH, tmp, fileName))
+		errc--;
+
+	if (stricmp(ODBCINST_INI, fileName) == 0)
+		return errc;
+
+	sprintf(tmp, "%d", comval->fetch_max);
+	if (!SQLWritePrivateProfileString(sectionName, INI_FETCH, tmp, fileName))
+		errc--;
 
 	sprintf(tmp, "%d", comval->disable_optimizer);
-	SQLWritePrivateProfileString(sectionName,
-								 INI_OPTIMIZER, tmp, fileName);
+	if (!SQLWritePrivateProfileString(sectionName, INI_OPTIMIZER, tmp, fileName))
+		errc--;
 
 	sprintf(tmp, "%d", comval->ksqo);
-	SQLWritePrivateProfileString(sectionName,
-								 INI_KSQO, tmp, fileName);
+	if (!SQLWritePrivateProfileString(sectionName, INI_KSQO, tmp, fileName))
+		errc--;
 
 	sprintf(tmp, "%d", comval->unique_index);
-	SQLWritePrivateProfileString(sectionName, INI_UNIQUEINDEX, tmp, fileName);
+	if (!SQLWritePrivateProfileString(sectionName, INI_UNIQUEINDEX, tmp, fileName))
+		errc--;
 	/*
 	 * Never update the onlyread from this module.
 	 */
@@ -912,49 +941,51 @@ writeDriverCommoninfo(const char *fileName, const char *sectionName,
 	}
 
 	sprintf(tmp, "%d", comval->use_declarefetch);
-	SQLWritePrivateProfileString(sectionName,
-								 INI_USEDECLAREFETCH, tmp, fileName);
+	if (!SQLWritePrivateProfileString(sectionName, INI_USEDECLAREFETCH, tmp, fileName))
+		errc--;
 
 	sprintf(tmp, "%d", comval->unknown_sizes);
-	SQLWritePrivateProfileString(sectionName,
-								 INI_UNKNOWNSIZES, tmp, fileName);
+	if (!SQLWritePrivateProfileString(sectionName, INI_UNKNOWNSIZES, tmp, fileName))
+		errc--;
 
 	sprintf(tmp, "%d", comval->text_as_longvarchar);
-	SQLWritePrivateProfileString(sectionName,
-								 INI_TEXTASLONGVARCHAR, tmp, fileName);
+	if (!SQLWritePrivateProfileString(sectionName, INI_TEXTASLONGVARCHAR, tmp, fileName))
+		errc--;
 
 	sprintf(tmp, "%d", comval->unknowns_as_longvarchar);
-	SQLWritePrivateProfileString(sectionName,
-							   INI_UNKNOWNSASLONGVARCHAR, tmp, fileName);
+	if (!SQLWritePrivateProfileString(sectionName, INI_UNKNOWNSASLONGVARCHAR, tmp, fileName))
+		errc--;
 
 	sprintf(tmp, "%d", comval->bools_as_char);
-	SQLWritePrivateProfileString(sectionName,
-								 INI_BOOLSASCHAR, tmp, fileName);
+	if (!SQLWritePrivateProfileString(sectionName, INI_BOOLSASCHAR, tmp, fileName))
+		errc--;
 
 	sprintf(tmp, "%d", comval->parse);
-	SQLWritePrivateProfileString(sectionName,
-								 INI_PARSE, tmp, fileName);
+	if (!SQLWritePrivateProfileString(sectionName, INI_PARSE, tmp, fileName))
+		errc--;
 
 	sprintf(tmp, "%d", comval->cancel_as_freestmt);
-	SQLWritePrivateProfileString(sectionName,
-								 INI_CANCELASFREESTMT, tmp, fileName);
+	if (!SQLWritePrivateProfileString(sectionName, INI_CANCELASFREESTMT, tmp, fileName))
+		errc--;
 
 	sprintf(tmp, "%d", comval->max_varchar_size);
-	SQLWritePrivateProfileString(sectionName,
-								 INI_MAXVARCHARSIZE, tmp, fileName);
+	if (!SQLWritePrivateProfileString(sectionName, INI_MAXVARCHARSIZE, tmp, fileName))
+		errc--;
 
 	sprintf(tmp, "%d", comval->max_longvarchar_size);
-	SQLWritePrivateProfileString(sectionName,
-								 INI_MAXLONGVARCHARSIZE, tmp, fileName);
+	if (!SQLWritePrivateProfileString(sectionName, INI_MAXLONGVARCHARSIZE, tmp, fileName))
+		errc--;
 
-	SQLWritePrivateProfileString(sectionName,
-	INI_EXTRASYSTABLEPREFIXES, comval->extra_systable_prefixes, fileName);
+	if (!SQLWritePrivateProfileString(sectionName, INI_EXTRASYSTABLEPREFIXES, comval->extra_systable_prefixes, fileName))
+		errc--;
 
 	/*
 	 * Never update the conn_setting from this module
 	 * SQLWritePrivateProfileString(sectionName, INI_CONNSETTINGS,
 	 * comval->conn_settings, fileName);
 	 */
+
+	return errc;
 }
 
 /*	This is for datasource based options only */
@@ -1299,27 +1330,29 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 }
 
 static void
-encode(const char *in, char *out)
+encode(const UCHAR *in, UCHAR *out)
 {
 	size_t i, ilen = strlen(in),
 				o = 0;
+	UCHAR	inc;
 
 	for (i = 0; i < ilen; i++)
 	{
-		if (in[i] == '+')
+		inc = in[i];
+		if (inc == '+')
 		{
 			sprintf(&out[o], "%%2B");
 			o += 3;
 		}
-		else if (isspace((UCHAR) in[i]))
+		else if (isspace(inc))
 			out[o++] = '+';
-		else if (!isalnum((UCHAR) in[i]))
+		else if (!isalnum(inc))
 		{
-			sprintf(&out[o], "%%%02x", (UCHAR) in[i]);
+			sprintf(&out[o], "%%%02x", inc);
 			o += 3;
 		}
 		else
-			out[o++] = in[i];
+			out[o++] = inc;
 	}
 	out[o++] = '\0';
 }
@@ -1347,22 +1380,24 @@ conv_from_hex(const UCHAR *s)
 }
 
 static void
-decode(const char *in, char *out)
+decode(const UCHAR *in, UCHAR *out)
 {
 	size_t i, ilen = strlen(in),
 				o = 0;
+	UCHAR	inc;
 
 	for (i = 0; i < ilen; i++)
 	{
-		if (in[i] == '+')
+		inc = in[i];
+		if (inc == '+')
 			out[o++] = ' ';
-		else if (in[i] == '%')
+		else if (inc == '%')
 		{
 			sprintf(&out[o++], "%c", conv_from_hex(&in[i]));
 			i += 2;
 		}
 		else
-			out[o++] = in[i];
+			out[o++] = inc;
 	}
 	out[o++] = '\0';
 }
