@@ -45,10 +45,10 @@ void		EN_log_error(const char *func, char *desc, EnvironmentClass *self);
 #define	EN_OV_ODBC2	1L
 #define	EN_CONN_POOLING	(1L<<1)
 #define	EN_is_odbc2(env) ((env->flag & EN_OV_ODBC2) != 0)
-#define	EN_is_odbc3(env) ((env->flag & EN_OV_ODBC2) == 0)
+#define	EN_is_odbc3(env) (env && (env->flag & EN_OV_ODBC2) == 0)
 #define EN_set_odbc2(env) (env->flag |= EN_OV_ODBC2)
 #define EN_set_odbc3(env) (env->flag &= ~EN_OV_ODBC2)
-#define	EN_is_pooling(env) ((env->flag & EN_CONN_POOLING) != 0)
+#define	EN_is_pooling(env) (env && (env->flag & EN_CONN_POOLING) != 0)
 #define	EN_set_pooling(env) (env->flag |= EN_CONN_POOLING)
 #define	EN_unset_pooling(env) (env->flag &= ~EN_CONN_POOLING)
 
@@ -94,10 +94,6 @@ void		EN_log_error(const char *func, char *desc, EnvironmentClass *self);
 #define DELETE_COMMON_CS
 #endif /* WIN_MULTITHREAD_SUPPORT */
 
-#ifdef	_HANDLE_ENLIST_IN_DTC_
-RETCODE	DtcOnRelease(void);
-#endif /* _HANDLE_ENLIST_IN_DTC_ */
-
 #ifdef	__cplusplus
 }
 #endif
