@@ -14,6 +14,7 @@
  */
 
 #include "psqlodbc.h"
+#include "misc.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -226,14 +227,13 @@ inquireHowToPrepare(const StatementClass *stmt)
 		}
 		if (stmt->multi_statement < 0)
 			PGAPI_NumParams((StatementClass *) stmt, &num_params);
-		if (stmt->multi_statement > 0) /* server-side prepare mechanism coundn' handle multi statement */
-			return PREPARE_BY_THE_DRIVER;
+		if (stmt->multi_statement > 0) /* would divide the query into multiple commands and apply V3 parse requests for each of them */
+			ret = PARSE_REQ_FOR_INFO;
 		else if (PROTOCOL_74(ci))
 		{
 			if (STMT_TYPE_SELECT == stmt->statement_type)
 			{
 				if (ci->drivers.use_declarefetch)
-					/* ret = PREPARE_BY_THE_DRIVER; */
 					return PARSE_REQ_FOR_INFO;
 				else if (SQL_CURSOR_FORWARD_ONLY != stmt->options.cursor_type)
 					ret = PARSE_REQ_FOR_INFO;
@@ -334,7 +334,7 @@ inolog("prepare_before_exec=%d srv=%d\n", prepare_before_exec, conn->connInfo.us
 	/*
 	 *	Dummy exection to get the column info.
 	 */ 
-	if (stmt->inaccurate_result && conn->connInfo.disallow_premature)
+	if (stmt->inaccurate_result && SC_is_parse_tricky(stmt))
 	{
 		BOOL		in_trans = CC_is_in_trans(conn);
 		BOOL		issued_begin = FALSE,
@@ -469,7 +469,7 @@ inolog("res->next=%p\n", kres);
 		EnvironmentClass *env = (EnvironmentClass *) (conn->henv);
 		const char *cmd = QR_get_command(res);
 
-		if (retval == SQL_SUCCESS && cmd && EN_is_odbc3(env))
+		if (retval == SQL_SUCCESS && cmd && env && EN_is_odbc3(env))
 		{
 			int     count;
 
@@ -672,7 +672,8 @@ inolog("ret=%d\n", ret);
 		}
 	}
 cleanup:
-	SC_forget_unnamed(stmt); /* unnamed plan is no longer reliable */
+	if (SQL_NEED_DATA != ret)
+		SC_forget_unnamed(stmt); /* unnamed plan is no longer reliable */
 	if (!SC_is_prepare_statement(stmt) && ONCE_DESCRIBED == stmt->prepared)
 		SC_set_prepared(stmt, NOT_YET_PREPARED);
 	if (start_stmt || SQL_ERROR == ret)
@@ -687,13 +688,22 @@ cleanup:
 	return ret;
 }
 
-static void
-SetInsertTable(StatementClass *stmt)
+void
+SC_setInsertedTable(StatementClass *stmt, RETCODE retval)
 {
 	const char *cmd = stmt->statement, *ptr;
 	ConnectionClass	*conn;
 	size_t	len;
 
+	if (STMT_TYPE_INSERT != stmt->statement_type)
+		return;
+	if (SQL_NEED_DATA == retval)
+		return;
+	conn = SC_get_conn(stmt);
+	if (PG_VERSION_GE(conn, 8.1)) /* lastval() is available */
+		return;
+	/*if (!CC_fake_mss(conn))
+		return;*/
 	while (isspace((UCHAR) *cmd)) cmd++;
 	if (!*cmd)
         	return;
@@ -711,9 +721,10 @@ SetInsertTable(StatementClass *stmt)
 	while (isspace((UCHAR) *(++cmd)));
 	if (!*cmd)
         	return;
-	conn = SC_get_conn(stmt);
 	NULL_THE_NAME(conn->schemaIns);
 	NULL_THE_NAME(conn->tableIns);
+	if (!SQL_SUCCEEDED(retval))
+		return;
 	ptr = NULL;
 	if (IDENTIFIER_QUOTE == *cmd)
 	{
@@ -731,7 +742,7 @@ SetInsertTable(StatementClass *stmt)
 	{
 		if (ptr = strchr(cmd + 1, '.'), NULL != ptr)
 		{
-			len = ptr - cmd - 1;
+			len = ptr - cmd;
 			STRN_TO_NAME(conn->schemaIns, cmd, len);
 			cmd = ptr + 1;
 			ptr = NULL; 
@@ -1047,14 +1058,7 @@ next_param_row:
 		goto next_param_row;
 cleanup:
 mylog("retval=%d\n", retval);
-	if (STMT_TYPE_INSERT == stmt->statement_type &&
-	    CC_fake_mss(conn) &&
-	    (SQL_SUCCESS == retval ||
-	     SQL_SUCCESS_WITH_INFO == retval)
-	   )
-	{
-		SetInsertTable(stmt);
-	}
+	SC_setInsertedTable(stmt, retval);
 #undef	return
 	if (stmt->internal)
 		retval = DiscardStatementSvp(stmt, retval, FALSE);
@@ -1374,6 +1378,7 @@ inolog("ipdopts=%p\n", ipdopts);
 	if (estmt->data_at_exec == 0)
 	{
 		BOOL	exec_end;
+		UWORD	flag = SC_is_with_hold(stmt) ? PODBC_WITH_HOLD : 0;
 
 		retval = Exec_with_parameters_resolved(estmt, &exec_end);
 		if (exec_end)
@@ -1382,7 +1387,7 @@ inolog("ipdopts=%p\n", ipdopts);
 			retval = dequeueNeedDataCallback(retval, stmt);
 			goto cleanup;
 		}
-		if (retval = PGAPI_Execute(estmt, 0), SQL_NEED_DATA != retval)
+		if (retval = PGAPI_Execute(estmt, flag), SQL_NEED_DATA != retval)
 		{
 			goto cleanup;
 		}
@@ -1433,12 +1438,7 @@ inolog("\n");
 inolog("return SQL_NEED_DATA\n");
 cleanup:
 #undef	return
-	if (STMT_TYPE_INSERT == stmt->statement_type &&
-	    CC_fake_mss(conn) &&
-	    (SQL_SUCCESS == retval ||
-	     SQL_SUCCESS_WITH_INFO == retval)
-	   )
-		SetInsertTable(stmt);
+	SC_setInsertedTable(stmt, retval);
 	if (stmt->internal)
 		retval = DiscardStatementSvp(stmt, retval, FALSE);
 	mylog("%s: returning %d\n", func, retval);
