@@ -15,6 +15,7 @@
 #include "psqlodbc.h"
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
 #include <time.h>
@@ -23,14 +24,18 @@
 #include <pwd.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <errno.h>
+#define	GENERAL_ERRNO		(errno)
+#define	GENERAL_ERRNO_SET(e)	(errno = e)
 #else
+#define	GENERAL_ERRNO		(GetLastError())
+#define	GENERAL_ERRNO_SET(e)	SetLastError(e)
 #include <process.h>			/* Byron: is this where Windows keeps def.
 								 * of getpid ? */
 #endif
 
 extern GLOBAL_VALUES globals;
-void		generate_filename(const char *, const char *, char *);
-
+void	generate_filename(const char *, const char *, char *);
 
 void
 generate_filename(const char *dirname, const char *prefix, char *filename)
@@ -60,6 +65,26 @@ generate_filename(const char *dirname, const char *prefix, char *filename)
 	return;
 }
 
+static void
+generate_homefile(const char *prefix, char *filename)
+{
+	char	dir[PATH_MAX];
+#ifdef	WIN32
+	const char *ptr;
+
+	dir[0] = '\0';
+	if (ptr=getenv("HOMEDRIVE"), NULL != ptr)
+		strcat(dir, ptr);
+	if (ptr=getenv("HOMEPATH"), NULL != ptr)
+		strcat(dir, ptr);
+#else
+	strcpy(dir, "~");	
+#endif /* WIN32 */
+	generate_filename(dir, prefix, filename);
+
+	return;
+}
+
 #if defined(WIN_MULTITHREAD_SUPPORT)
 static	CRITICAL_SECTION	qlog_cs, mylog_cs;
 #elif defined(POSIX_MULTITHREAD_SUPPORT)
@@ -129,8 +154,11 @@ mylog(const char *fmt,...)
 {
 	va_list		args;
 	char		filebuf[80];
+	int		gerrno;
 
 	if (!mylog_on)	return;
+
+	gerrno = GENERAL_ERRNO;
 	ENTER_MYLOG_CS;
 #ifdef	LOGGING_PROCESS_TIME
 	if (!start_time)
@@ -142,38 +170,47 @@ mylog(const char *fmt,...)
 	{
 		generate_filename(MYLOGDIR, MYLOGFILE, filebuf);
 		MLOGFP = fopen(filebuf, PG_BINARY_A);
+		if (!MLOGFP)
+		{
+			generate_homefile(MYLOGFILE, filebuf);
+			MLOGFP = fopen(filebuf, PG_BINARY_A);
+		}
 		if (MLOGFP)
 			setbuf(MLOGFP, NULL);
+		else
+			mylog_on = 0;
 	}
 
-#ifdef	WIN_MULTITHREAD_SUPPORT
-#ifdef	WIN32
 	if (MLOGFP)
-#ifdef	LOGGING_PROCESS_TIME
 	{
+#ifdef	WIN_MULTITHREAD_SUPPORT
+#ifdef	LOGGING_PROCESS_TIME
 		DWORD	proc_time = timeGetTime() - start_time;
 		fprintf(MLOGFP, "[%u-%d.%03d]", GetCurrentThreadId(), proc_time / 1000, proc_time % 1000);
-	}
 #else
 		fprintf(MLOGFP, "[%u]", GetCurrentThreadId());
 #endif /* LOGGING_PROCESS_TIME */
-#endif /* WIN32 */
 #endif /* WIN_MULTITHREAD_SUPPORT */
 #if defined(POSIX_MULTITHREAD_SUPPORT)
-	if (MLOGFP)
 		fprintf(MLOGFP, "[%lu]", pthread_self());
 #endif /* POSIX_MULTITHREAD_SUPPORT */
-	if (MLOGFP)
 		vfprintf(MLOGFP, fmt, args);
+	}
 
 	va_end(args);
 	LEAVE_MYLOG_CS;
+	GENERAL_ERRNO_SET(gerrno);
 }
 void
 forcelog(const char *fmt,...)
 {
+	static BOOL	force_on = TRUE;
 	va_list		args;
 	char		filebuf[80];
+	int		gerrno = GENERAL_ERRNO;
+
+	if (!force_on)
+		return;
 
 	ENTER_MYLOG_CS;
 	va_start(args, fmt);
@@ -184,13 +221,20 @@ forcelog(const char *fmt,...)
 		MLOGFP = fopen(filebuf, PG_BINARY_A);
 		if (MLOGFP)
 			setbuf(MLOGFP, NULL);
-	}
-	if (!MLOGFP)
-	{
-		generate_filename("C:\\podbclog", MYLOGFILE, filebuf);
-		MLOGFP = fopen(filebuf, PG_BINARY_A);
+		if (!MLOGFP)
+		{
+			generate_homefile(MYLOGFILE, filebuf);
+			MLOGFP = fopen(filebuf, PG_BINARY_A);
+		}
+		if (!MLOGFP)
+		{
+			generate_filename("C:\\podbclog", MYLOGFILE, filebuf);
+			MLOGFP = fopen(filebuf, PG_BINARY_A);
+		}
 		if (MLOGFP)
 			setbuf(MLOGFP, NULL);
+		else
+			force_on = FALSE;
 	}
 	if (MLOGFP)
 	{
@@ -212,6 +256,7 @@ forcelog(const char *fmt,...)
 	}
 	va_end(args);
 	LEAVE_MYLOG_CS;
+	GENERAL_ERRNO_SET(gerrno);
 }
 static void mylog_initialize()
 {
@@ -244,25 +289,45 @@ qlog(char *fmt,...)
 {
 	va_list		args;
 	char		filebuf[80];
+	int		gerrno;
 
 	if (!qlog_on)	return;
 
+	gerrno = GENERAL_ERRNO;
 	ENTER_QLOG_CS;
+#ifdef	LOGGING_PROCESS_TIME
+	if (!start_time)
+		start_time = timeGetTime();
+#endif /* LOGGING_PROCESS_TIME */
 	va_start(args, fmt);
 
 	if (!QLOGFP)
 	{
 		generate_filename(QLOGDIR, QLOGFILE, filebuf);
 		QLOGFP = fopen(filebuf, PG_BINARY_A);
+		if (!QLOGFP)
+		{
+			generate_homefile(QLOGFILE, filebuf);
+			QLOGFP = fopen(filebuf, PG_BINARY_A);
+		}
 		if (QLOGFP)
 			setbuf(QLOGFP, NULL);
+		else
+			qlog_on = 0;
 	}
 
 	if (QLOGFP)
+	{
+#ifdef	LOGGING_PROCESS_TIME
+		DWORD	proc_time = timeGetTime() - start_time;
+		fprintf(QLOGFP, "[%d.%03d]", proc_time / 1000, proc_time % 1000);
+#endif /* LOGGING_PROCESS_TIME */
 		vfprintf(QLOGFP, fmt, args);
+	}
 
 	va_end(args);
 	LEAVE_QLOG_CS;
+	GENERAL_ERRNO_SET(gerrno);
 }
 static void qlog_initialize()
 {
