@@ -35,6 +35,7 @@
 #include "connection.h"
 #include "statement.h"
 #include "qresult.h"
+#include "loadlib.h"
 
 #if (ODBCVER < 0x0300)
 RETCODE		SQL_API
@@ -295,7 +296,7 @@ SQLDisconnect(HDBC ConnectionHandle)
 
 	mylog("[%s for %p]", func, ConnectionHandle);
 #ifdef	_HANDLE_ENLIST_IN_DTC_
-	DtcOnDisconnect(conn); /* must be called without holding the connection lock */
+	CALL_DtcOnDisconnect(conn); /* must be called without holding the connection lock */
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
 	ENTER_CONN_CS(conn);
 	CC_clear_error(conn);
@@ -1104,7 +1105,11 @@ SQLExtendedFetch(
 				 HSTMT hstmt,
 				 SQLUSMALLINT fFetchType,
 				 SQLLEN irow,
+#ifdef WITH_UNIXODBC
+				 SQLROWSETSIZE *pcrow,
+#else
 				 SQLULEN *pcrow,
+#endif /* WITH_UNIXODBC */
 				 SQLUSMALLINT *rgfRowStatus)
 {
 	RETCODE	ret;
@@ -1114,7 +1119,17 @@ SQLExtendedFetch(
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
 	StartRollbackState(stmt);
+#ifdef WITH_UNIXODBC
+	{
+		SQLULEN	retrieved;
+
+		ret = PGAPI_ExtendedFetch(hstmt, fFetchType, irow, &retrieved, rgfRowStatus, 0, SC_get_ARDF(stmt)->size_of_rowset_odbc2);
+		if (pcrow)
+			*pcrow = retrieved;
+	}
+#else
 	ret = PGAPI_ExtendedFetch(hstmt, fFetchType, irow, pcrow, rgfRowStatus, 0, SC_get_ARDF(stmt)->size_of_rowset_odbc2);
+#endif /* WITH_UNIXODBC */
 	stmt->transition_status = 7;
 	ret = DiscardStatementSvp(stmt, ret, FALSE);
 	LEAVE_STMT_CS(stmt);
