@@ -275,18 +275,18 @@ getColInfo(COL_INFO *col_info, FIELD_INFO *fi, int k)
 
 inolog("getColInfo non-manual result\n");
 	fi->dquote = TRUE;
-	STR_TO_NAME(fi->column_name, QR_get_value_backend_row(col_info->result, k, COLUMNS_COLUMN_NAME));
+	STR_TO_NAME(fi->column_name, QR_get_value_backend_text(col_info->result, k, COLUMNS_COLUMN_NAME));
 
-	fi->columntype = atoi(QR_get_value_backend_row(col_info->result, k, COLUMNS_FIELD_TYPE));
-	fi->column_size = atoi(QR_get_value_backend_row(col_info->result, k, COLUMNS_PRECISION));
-	fi->length = atoi(QR_get_value_backend_row(col_info->result, k, COLUMNS_LENGTH));
-	if (str = QR_get_value_backend_row(col_info->result, k, COLUMNS_SCALE), str)
+	fi->columntype = (OID) QR_get_value_backend_int(col_info->result, k, COLUMNS_FIELD_TYPE, NULL);
+	fi->column_size = QR_get_value_backend_int(col_info->result, k, COLUMNS_PRECISION, NULL);
+	fi->length = QR_get_value_backend_int(col_info->result, k, COLUMNS_LENGTH, NULL);
+	if (str = QR_get_value_backend_text(col_info->result, k, COLUMNS_SCALE), str)
 		fi->decimal_digits = atoi(str);
 	else
 		fi->decimal_digits = -1;
-	fi->nullable = atoi(QR_get_value_backend_row(col_info->result, k, COLUMNS_NULLABLE));
-	fi->display_size = atoi(QR_get_value_backend_row(col_info->result, k, COLUMNS_DISPLAY_SIZE));
-	fi->auto_increment = atoi(QR_get_value_backend_row(col_info->result, k, COLUMNS_AUTO_INCREMENT));
+	fi->nullable = QR_get_value_backend_int(col_info->result, k, COLUMNS_NULLABLE, NULL);
+	fi->display_size = QR_get_value_backend_int(col_info->result, k, COLUMNS_DISPLAY_SIZE, NULL);
+	fi->auto_increment = QR_get_value_backend_int(col_info->result, k, COLUMNS_AUTO_INCREMENT, NULL);
 }
 
 
@@ -295,7 +295,7 @@ searchColInfo(COL_INFO *col_info, FIELD_INFO *fi)
 {
 	int			k,
 				cmp, attnum;
-	char	   *col;
+	const char	   *col;
 
 inolog("searchColInfo num_cols=%d col=%s\n", QR_get_num_cached_tuples(col_info->result), PRINT_NAME(fi->column_name));
 	if (fi->attnum < 0)
@@ -304,7 +304,7 @@ inolog("searchColInfo num_cols=%d col=%s\n", QR_get_num_cached_tuples(col_info->
 	{
 		if (fi->attnum > 0)
 		{
-			attnum = atoi(QR_get_value_backend_row(col_info->result, k, COLUMNS_PHYSICAL_NUMBER));
+			attnum = QR_get_value_backend_int(col_info->result, k, COLUMNS_PHYSICAL_NUMBER, NULL);
 inolog("searchColInfo %d attnum=%d\n", k, attnum);
 			if (attnum == fi->attnum)
 			{
@@ -315,7 +315,7 @@ inolog("searchColInfo %d attnum=%d\n", k, attnum);
 		}
 		else if (NAME_IS_VALID(fi->column_name))
 		{
-			col = QR_get_value_backend_row(col_info->result, k, COLUMNS_COLUMN_NAME);
+			col = QR_get_value_backend_text(col_info->result, k, COLUMNS_COLUMN_NAME);
 inolog("searchColInfo %d col=%s\n", k, col);
 			if (fi->dquote)
 				cmp = strcmp(col, GET_NAME(fi->column_name));
@@ -378,7 +378,7 @@ static BOOL CheckHasOids(StatementClass * stmt)
 		stmt->num_key_fields = PG_NUM_NORMAL_KEYS;
 		if (1 == QR_get_num_total_tuples(res))
 		{
-			char *value = QR_get_value_backend_row(res, 0, 0);
+			const char *value = QR_get_value_backend_text(res, 0, 0);
 			if (value && ('f' == *value || '0' == *value))
 			{
 				hasoids = FALSE;
@@ -394,7 +394,7 @@ static BOOL CheckHasOids(StatementClass * stmt)
 				STR_TO_NAME(ti->bestqual, query);
 			}
 			TI_set_hasoids_checked(ti);
-			ti->table_oid = strtoul(QR_get_value_backend_row(res, 0, 1), NULL, 10);
+			ti->table_oid = (OID) strtoul(QR_get_value_backend_text(res, 0, 1), NULL, 10);
 		}
 		QR_Destructor(res);
 		res = NULL;
@@ -405,9 +405,9 @@ static BOOL CheckHasOids(StatementClass * stmt)
 			if (QR_command_maybe_successful(res) && QR_get_num_total_tuples(res) > 0)
 			{
 				foundKey = TRUE;
-				STR_TO_NAME(ti->bestitem, QR_get_value_backend_row(res, 0, 0));
+				STR_TO_NAME(ti->bestitem, QR_get_value_backend_text(res, 0, 0));
 				sprintf(query, "\"%s\" = %%", SAFE_NAME(ti->bestitem));
-				if (PG_TYPE_INT4 == atoi(QR_get_value_backend_row(res, 0, 1)))
+				if (PG_TYPE_INT4 == (OID) QR_get_value_backend_int(res, 0, 1, NULL))
 					strcat(query, "d");
 				else
 					strcat(query, "u");
@@ -451,6 +451,73 @@ static BOOL increaseNtab(StatementClass *stmt, const char *func)
 	stmt->ntab++;
 	return TRUE;
 }
+	
+static void setNumFields(IRDFields *irdflds, size_t numFields)
+{
+	FIELD_INFO	**fi = irdflds->fi;
+	size_t		nfields = irdflds->nfields;
+
+	if (numFields < nfields)
+	{
+		int	i;
+
+		for (i = (int) numFields; i < (int) nfields; i++)
+		{
+			if (fi[i])
+				fi[i]->flag = 0;
+		}
+	}
+	irdflds->nfields = (UInt4) numFields;
+}
+
+void SC_initialize_cols_info(StatementClass *stmt, BOOL DCdestroy, BOOL parseReset)
+{
+	IRDFields	*irdflds = SC_get_IRDF(stmt);
+
+	/* Free the parsed table information */
+	if (stmt->ti)
+	{
+		TI_Destructor(stmt->ti, stmt->ntab);
+		free(stmt->ti);
+		stmt->ti = NULL;
+	}
+	stmt->ntab = 0;
+	if (DCdestroy) /* Free the parsed field information */
+		DC_Destructor((DescriptorClass *) SC_get_IRD(stmt));
+	else
+		setNumFields(irdflds, 0);
+	if (parseReset)
+	{
+		stmt->parse_status = STMT_PARSE_NONE;
+		stmt->updatable = FALSE;
+	}
+}
+
+static BOOL allocateFields(IRDFields *irdflds, size_t sizeRequested)
+{
+	FIELD_INFO	**fi = irdflds->fi;
+	size_t		alloc_size, incr_size;
+
+	if (sizeRequested <= irdflds->allocated)
+		return TRUE;
+	alloc_size = (0 != irdflds->allocated ? irdflds->allocated : FLD_INCR);
+	for (; alloc_size < sizeRequested; alloc_size *= 2)
+			;
+	incr_size = sizeof(FIELD_INFO *) * (alloc_size - irdflds->allocated);
+
+	fi = (FIELD_INFO **) realloc(fi, alloc_size * sizeof(FIELD_INFO *));
+	if (!fi)
+	{
+		irdflds->fi = NULL;
+		irdflds->allocated = irdflds->nfields = 0;
+		return FALSE;
+	}
+	memset(&fi[irdflds->allocated], 0, incr_size);
+	irdflds->fi = fi;
+	irdflds->allocated = (SQLSMALLINT) alloc_size;
+
+	return TRUE;
+}
 
 static void xxxxx(FIELD_INFO *fi, QResultClass *res, int i)
 {
@@ -512,25 +579,19 @@ mylog("ColAttSet in\n");
 	fi = irdflds->fi;
 	if (num_fields > (int) irdflds->allocated)
 	{
-		fi = (FIELD_INFO **) realloc(fi, num_fields * sizeof(FIELD_INFO *));
-		if (!fi)
-		{
-			irdflds->fi = NULL;
-			irdflds->allocated = irdflds->nfields = 0;
+		if (!allocateFields(irdflds, num_fields))
 			return FALSE;
-		}
-		irdflds->fi = fi;
-		irdflds->allocated = num_fields;
-		for (i = irdflds->nfields; i < num_fields; i++)
-			fi[i] = NULL;
-		irdflds->nfields = num_fields;
+		fi = irdflds->fi;
 	}
+	setNumFields(irdflds, num_fields);
 	updatable = rti ? TI_is_updatable(rti) : FALSE;
 mylog("updatable=%d tab=%d fields=%d", updatable, stmt->ntab, num_fields);
 	if (updatable)
 	{
 		if (1 != stmt->ntab)
 			updatable = FALSE;
+		else if (SC_has_join(stmt))
+			updatable = FALSE;
 		else
 		{
 			OID	greloid;
@@ -632,7 +693,7 @@ COL_INFO **coli)
 					if (QR_get_num_total_tuples(res) == 1)
 					{
 						tblFound = TRUE;
-						STR_TO_NAME(*schema_name, QR_get_value_backend_row(res, 0, 0));
+						STR_TO_NAME(*schema_name, QR_get_value_backend_text(res, 0, 0));
 					}
 				}
 				QR_Destructor(res);
@@ -749,7 +810,7 @@ inolog("fi=%p greloid=%d col_info=%p\n", wti, greloid, wti->col_info);
 	}
 	if (found)
 		goto cleanup;
-	else
+	else if (0 != greloid || NAME_IS_VALID(wti->table_name))
 	{
 		RETCODE		result;
 		StatementClass	*col_stmt;
@@ -816,15 +877,15 @@ inolog("fi=%p greloid=%d col_info=%p\n", wti, greloid, wti->col_info);
 			if (res && QR_get_num_cached_tuples(res) > 0)
 			{
 				if (!greloid)
-					greloid = strtoul(QR_get_value_backend_row(res, 0, COLUMNS_TABLE_OID), NULL, 10);
+					greloid = (OID) strtoul(QR_get_value_backend_text(res, 0, COLUMNS_TABLE_OID), NULL, 10);
 				if (!wti->table_oid)
 					wti->table_oid = greloid;
 				if (NAME_IS_NULL(wti->schema_name))
 					STR_TO_NAME(wti->schema_name, 
-						QR_get_value_backend_row(res, 0, COLUMNS_SCHEMA_NAME));
+						QR_get_value_backend_text(res, 0, COLUMNS_SCHEMA_NAME));
 				if (NAME_IS_NULL(wti->table_name))
 					STR_TO_NAME(wti->table_name, 
-						QR_get_value_backend_row(res, 0, COLUMNS_TABLE_NAME));
+						QR_get_value_backend_text(res, 0, COLUMNS_TABLE_NAME));
 			}
 inolog("#2 %p->table_name=%s(%u)\n", wti, PRINT_NAME(wti->table_name), wti->table_oid);
 			/*
@@ -849,7 +910,7 @@ inolog("#2 %p->table_name=%s(%u)\n", wti, PRINT_NAME(wti->table_name), wti->tabl
 			conn->ntables++;
 
 if (res && QR_get_num_cached_tuples(res) > 0)
-inolog("oid item == %s\n", QR_get_value_backend_row(res, 0, 3));
+inolog("oid item == %s\n", QR_get_value_backend_text(res, 0, 3));
 
 			mylog("Created col_info table='%s', ntables=%d\n", PRINT_NAME(wti->table_name), conn->ntables);
 			/* Associate a table from the statement with a SQLColumn info */
@@ -867,15 +928,15 @@ cleanup:
 		if (res && QR_get_num_cached_tuples(res) > 0)
 		{
 			if (!greloid)
-				greloid = strtoul(QR_get_value_backend_row(res, 0, COLUMNS_TABLE_OID), NULL, 10);
+				greloid = (OID) strtoul(QR_get_value_backend_text(res, 0, COLUMNS_TABLE_OID), NULL, 10);
 			if (!wti->table_oid)
 				wti->table_oid = greloid;
 			if (NAME_IS_NULL(wti->schema_name))
 				STR_TO_NAME(wti->schema_name, 
-					QR_get_value_backend_row(res, 0, COLUMNS_SCHEMA_NAME));
+					QR_get_value_backend_text(res, 0, COLUMNS_SCHEMA_NAME));
 			if (NAME_IS_NULL(wti->table_name))
 				STR_TO_NAME(wti->table_name, 
-					QR_get_value_backend_row(res, 0, COLUMNS_TABLE_NAME));
+					QR_get_value_backend_text(res, 0, COLUMNS_TABLE_NAME));
 		}
 inolog("#1 %p->table_name=%s(%u)\n", wti, PRINT_NAME(wti->table_name), wti->table_oid);
 		if (colatt /* SQLColAttribute case */
@@ -920,10 +981,10 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 				k = 0,
 				n,
 				blevel = 0, old_blevel, subqlevel = 0,
-				nfields_old, allocated_size, new_size;
+				allocated_size, new_size;
 	FIELD_INFO **fi, *wfi;
 	TABLE_INFO **ti, *wti;
-	char		parse;
+	char		parse, maybe_join = 0;
 	ConnectionClass *conn = SC_get_conn(stmt);
 	IRDFields	*irdflds = SC_get_IRDF(stmt);
 	BOOL		updatable = TRUE;
@@ -942,17 +1003,10 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 	ti = stmt->ti;
 
 	allocated_size = irdflds->allocated;
-	nfields_old = irdflds->nfields;
-	irdflds->nfields = 0;
-	if (stmt->ntab > 0)
-	{
-		TI_Destructor(stmt->ti, stmt->ntab);
-		free(stmt->ti);
-		stmt->ti = NULL;
-		stmt->ntab = 0;
-	}
+	SC_initialize_cols_info(stmt, FALSE, TRUE);
 	stmt->from_pos = -1;
 	stmt->where_pos = -1;
+#define	return	DONT_CALL_RETURN_FROM_HERE???
 
 	while (pptr = ptr, (ptr = getNextToken(conn->ccsc, CC_get_escape(conn), pptr, token, sizeof(token), &delim, &quote, &dquote, &numeric)) != NULL)
 	{
@@ -979,7 +1033,7 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 					mylog("INTO\n");
 					stmt->statement_type = STMT_TYPE_CREATE;
 					SC_set_parse_status(stmt, STMT_PARSE_FATAL);
-					return FALSE;
+					goto cleanup;
 				}
 				else if (!stricmp(token, "from"))
 				{
@@ -1126,22 +1180,18 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 				if (irdflds->nfields >= allocated_size)
 				{
 					mylog("reallocing at nfld=%d\n", irdflds->nfields);
-					new_size = (irdflds->nfields / FLD_INCR + 1) * FLD_INCR;
-					fi = (FIELD_INFO **) realloc(fi, new_size * sizeof(FIELD_INFO *));
-					if (!fi)
+					new_size = irdflds->nfields + 1;
+					if (!allocateFields(irdflds, new_size))
 					{
 						SC_set_parse_status(stmt, STMT_PARSE_FATAL);
 						SC_set_error(stmt, STMT_NO_MEMORY_ERROR, "PGAPI_AllocStmt failed in parse_statement for FIELD_INFO.", func);
-						return FALSE;
+						goto cleanup;
 					}
-					allocated_size = new_size;
-					irdflds->fi = fi;
-					irdflds->allocated = allocated_size;
+					fi = irdflds->fi;
+					allocated_size = irdflds->allocated;
 				}
 
-				wfi = NULL;
-				if (irdflds->nfields < nfields_old)
-					wfi = fi[irdflds->nfields];
+				wfi = fi[irdflds->nfields];
 				if (wfi)
 					fi_reuse = TRUE;
 				else
@@ -1150,7 +1200,7 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 				{
 					SC_set_parse_status(stmt, STMT_PARSE_FATAL);
 					SC_set_error(stmt, STMT_NO_MEMORY_ERROR, "PGAPI_AllocStmt failed in parse_statement for FIELD_INFO(2).", func);
-					return FALSE;
+					goto cleanup;
 				}
 
 				/* Initialize the field info */
@@ -1265,13 +1315,16 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 			mylog("*** setting expression\n");
 		} /* in_select end */
 
-		if (in_from)
+		if (in_from || in_where)
 		{
-			if (token[0] == ';')
+			if (token[0] == ';') /* end of the first command */
 			{
-				in_from = FALSE;
+				in_select = in_from = in_where = in_table = FALSE;
 				break;
 			}
+		}
+		if (in_from)
+		{
 			switch (token[0])
 			{
 				case '\0':
@@ -1282,6 +1335,8 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 			}
 			if (out_table && !in_table) /* new table */
 			{
+				in_dot = FALSE;
+				maybe_join = 0;
 				if (!dquote)
 				{
 					if (token[0] == '(' ||
@@ -1292,16 +1347,24 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 				if (!increaseNtab(stmt, func))
 				{
 					SC_set_parse_status(stmt, STMT_PARSE_FATAL);
-					return FALSE;
+					goto cleanup;
 				}
 
 				ti = stmt->ti;
 				wti = ti[stmt->ntab - 1];
-				STR_TO_NAME(wti->table_name, token);
-				lower_the_name(GET_NAME(wti->table_name), conn, dquote);
+				if (dquote || stricmp(token, "select"))
+				{
+					STR_TO_NAME(wti->table_name, token);
+					lower_the_name(GET_NAME(wti->table_name), conn, dquote);
+				}
+				else
+				{
+					NULL_THE_NAME(wti->table_name);
+					TI_no_updatable(wti);
+				}
 				mylog("got table = '%s'\n", PRINT_NAME(wti->table_name));
 
-				if (delim == ',')
+				if (0 == blevel && delim == ',')
 				{
 					out_table = TRUE;
 					mylog("more than 1 tables\n");
@@ -1311,16 +1374,49 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 					out_table = FALSE;
 					in_table = TRUE;
 				}
-				in_dot = FALSE;
 				continue;
 			}
-
-			if (!dquote && stricmp(token, "JOIN") == 0)
+			if (0 != blevel)
+				continue;
+			/* out_table is FALSE here */
+			if (!dquote && !in_dot)
 			{
-				in_table = FALSE;
-				out_table = TRUE;
-				continue;
+				if (')' == token[0])
+					continue;
+				if (stricmp(token, "LEFT") == 0 ||
+			    	    stricmp(token, "RIGHT") == 0 ||
+			    	    stricmp(token, "OUTER") == 0 ||
+			    	    stricmp(token, "FULL") == 0)
+				{
+					maybe_join = 1;
+					in_table = FALSE;
+					continue;
+				}
+				else if (stricmp(token, "INNER") == 0 ||
+					 stricmp(token, "CROSS") == 0)
+				{
+					maybe_join = 2;
+					in_table = FALSE;
+					continue;
+				}
+				else if (stricmp(token, "JOIN") == 0)
+				{
+					in_table = FALSE;
+					out_table = TRUE;
+					switch (maybe_join)
+					{
+						case 1:
+							SC_set_outer_join(stmt);
+							break;
+						case 2:
+							SC_set_inner_join(stmt);
+							break;
+					}
+					maybe_join = 0;
+					continue;
+				}
 			}
+			maybe_join = 0;
 			if (in_table)
 			{
 				wti = ti[stmt->ntab - 1];
@@ -1339,13 +1435,9 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 				}
 				if (dquote || stricmp(token, "as"))
 				{
-					if (stricmp(token, "LEFT") == 0 ||
-					    stricmp(token, "RIGHT") == 0 ||
-					    stricmp(token, "OUTER") == 0 ||
-					    stricmp(token, "FULL") == 0 ||
-					    stricmp(token, "ON") == 0)
+					if (!dquote)
 					{
-						if (!dquote)
+						if (stricmp(token, "ON") == 0)
 						{
 							in_table = FALSE;
 							continue;
@@ -1422,7 +1514,7 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 							SC_set_parse_status(stmt, STMT_PARSE_FATAL);
 							SC_set_error(stmt, STMT_EXEC_ERROR, "duplicated Table name", func);
 							stmt->updatable = FALSE;
-							return FALSE;
+							goto cleanup;
 						}
 					}
 				}
@@ -1457,7 +1549,7 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 	if (0 == stmt->ntab)
 	{
 		SC_set_parse_status(stmt, STMT_PARSE_FATAL);
-		return	FALSE;
+		goto cleanup;
 	}
 
 	for (i = 0; i < (int) irdflds->nfields; i++)
@@ -1492,7 +1584,9 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 			break;
 	}
 	if (STMT_PARSE_FATAL == SC_parsed_status(stmt))
-		return FALSE;
+	{
+		goto cleanup;
+	}
 
 	mylog("Done PG_Columns\n");
 
@@ -1538,25 +1632,22 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 			increased_cols = total_cols - 1;
 
 			/* Allocate some more field pointers if necessary */
-			/* allocated_size = ((irdflds->nfields - 1) / FLD_INCR + 1) * FLD_INCR; */
 			new_size = irdflds->nfields + increased_cols;
 
 			mylog("k=%d, increased_cols=%d, allocated_size=%d, new_size=%d\n", k, increased_cols, allocated_size, new_size);
 
 			if (new_size > allocated_size)
 			{
-				int			new_alloc = ((new_size / FLD_INCR) + 1) * FLD_INCR;
+				int	new_alloc = new_size;
 
 				mylog("need more cols: new_alloc = %d\n", new_alloc);
-				fi = (FIELD_INFO **) realloc(fi, new_alloc * sizeof(FIELD_INFO *));
-				if (!fi)
+				if (!allocateFields(irdflds, new_alloc))
 				{
 					SC_set_parse_status(stmt, STMT_PARSE_FATAL);
-					return FALSE;
+					goto cleanup;
 				}
-				allocated_size = new_alloc;
-				irdflds->fi = fi;
-				irdflds->allocated = allocated_size;
+				fi = irdflds->fi;
+				allocated_size = irdflds->allocated;
 			}
 
 			/*
@@ -1599,7 +1690,7 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 						if (fi[n + i] == NULL)
 						{
 							SC_set_parse_status(stmt, STMT_PARSE_FATAL);
-							return FALSE;
+							goto cleanup;
 						}
 						reuse = FALSE;
 					}
@@ -1668,6 +1759,14 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 	}
 
 	stmt->updatable = updatable;
+cleanup:
+#undef	return
+	if (STMT_PARSE_FATAL == SC_parsed_status(stmt))
+	{
+		SC_initialize_cols_info(stmt, FALSE, FALSE);
+		parse = FALSE;
+	}
+
 	mylog("done parse_statement: parse=%d, parse_status=%d\n", parse, SC_parsed_status(stmt));
 	return parse;
 }
