@@ -15,6 +15,7 @@
  */
 
 #include "psqlodbc.h"
+#include "misc.h"
 
 #if (ODBCVER >= 0x0300)
 #include <stdio.h>
@@ -26,6 +27,7 @@
 #include "descriptor.h"
 #include "qresult.h"
 #include "pgapifunc.h"
+#include "loadlib.h"
 
 
 /*	SQLError -> SQLDiagRec */
@@ -123,8 +125,7 @@ PGAPI_GetDiagField(SQLSMALLINT HandleType, SQLHANDLE Handle,
 					ret = PGAPI_EnvError(Handle, RecNumber,
                         			NULL, NULL, NULL,
 						0, NULL, 0);
-					if (SQL_SUCCESS == ret ||
-					    SQL_SUCCESS_WITH_INFO == ret)
+					if (SQL_SUCCEEDED(ret))
 					{
 						*((SQLINTEGER *) DiagInfoPtr) = 1;
 					}
@@ -189,8 +190,7 @@ PGAPI_GetDiagField(SQLSMALLINT HandleType, SQLHANDLE Handle,
 					ret = PGAPI_ConnectError(Handle, RecNumber,
                         			NULL, NULL, NULL,
 						0, NULL, 0);
-					if (SQL_SUCCESS == ret ||
-					    SQL_SUCCESS_WITH_INFO == ret)
+					if (SQL_SUCCEEDED(ret))
 					{
 						*((SQLINTEGER *) DiagInfoPtr) = 1;
 					}
@@ -476,9 +476,9 @@ ARDSetField(DescriptorClass *desc, SQLSMALLINT RecNumber,
 		SQLSMALLINT FieldIdentifier, PTR Value, SQLINTEGER BufferLength)
 {
 	RETCODE		ret = SQL_SUCCESS;
-	PTR		tptr;
 	ARDFields	*opts = (ARDFields *) (desc + 1);
 	SQLSMALLINT	row_idx;
+	BOOL		unbind = TRUE;
 
 	switch (FieldIdentifier)
 	{
@@ -519,12 +519,7 @@ ARDSetField(DescriptorClass *desc, SQLSMALLINT RecNumber,
 				bookmark->buffer = Value;
 				break;
 			case SQL_DESC_INDICATOR_PTR:
-				tptr = bookmark->used;
-				if (Value != tptr)
-				{
-					DC_set_error(desc, DESC_INVALID_DESCRIPTOR_IDENTIFIER, "INDICATOR != OCTET_LENGTH_PTR"); 
-					ret = SQL_ERROR;
-				}
+				bookmark->indicator = Value;
 				break;
 			case SQL_DESC_OCTET_LENGTH_PTR:
 				bookmark->used = Value;
@@ -539,7 +534,6 @@ ARDSetField(DescriptorClass *desc, SQLSMALLINT RecNumber,
 	switch (FieldIdentifier)
 	{
 		case SQL_DESC_TYPE:
-			reset_a_column_binding(opts, RecNumber);
 			opts->bindings[row_idx].returntype = CAST_PTR(SQLSMALLINT, Value);
 			break;
 		case SQL_DESC_DATETIME_INTERVAL_CODE:
@@ -568,17 +562,15 @@ ARDSetField(DescriptorClass *desc, SQLSMALLINT RecNumber,
 			opts->bindings[row_idx].returntype = CAST_PTR(SQLSMALLINT, Value);
 			break;
 		case SQL_DESC_DATA_PTR:
+			unbind = TRUE;
 			opts->bindings[row_idx].buffer = Value;
 			break;
 		case SQL_DESC_INDICATOR_PTR:
-			tptr = opts->bindings[row_idx].used;
-			if (Value != tptr)
-			{
-				ret = SQL_ERROR;
-				DC_set_error(desc, DESC_INVALID_DESCRIPTOR_IDENTIFIER, "INDICATOR != OCTET_LENGTH_PTR"); 
-			}
+			unbind = TRUE;
+			opts->bindings[row_idx].indicator = Value;
 			break;
 		case SQL_DESC_OCTET_LENGTH_PTR:
+			unbind = TRUE;
 			opts->bindings[row_idx].used = Value;
 			break;
 		case SQL_DESC_OCTET_LENGTH:
@@ -598,6 +590,8 @@ ARDSetField(DescriptorClass *desc, SQLSMALLINT RecNumber,
 			DC_set_error(desc, DESC_INVALID_DESCRIPTOR_IDENTIFIER,
 				"invalid descriptor identifier"); 
 	}
+	if (unbind)
+		opts->bindings[row_idx].buffer = NULL;
 	return ret;
 }
 
@@ -651,9 +645,11 @@ static RETCODE SQL_API
 APDSetField(DescriptorClass *desc, SQLSMALLINT RecNumber,
 		SQLSMALLINT FieldIdentifier, PTR Value, SQLINTEGER BufferLength)
 {
+	CSTR		func = "APDSetField";
 	RETCODE		ret = SQL_SUCCESS;
 	APDFields	*opts = (APDFields *) (desc + 1);
 	SQLSMALLINT	para_idx;
+	BOOL		unbind = TRUE;
 
 	switch (FieldIdentifier)
 	{
@@ -681,14 +677,14 @@ APDSetField(DescriptorClass *desc, SQLSMALLINT RecNumber,
 	}
 	if (RecNumber <=0)
 	{
-inolog("APDSetField RecN=%d allocated=%d\n", RecNumber, opts->allocated);
+inolog("%s RecN=%d allocated=%d\n", func, RecNumber, opts->allocated);
 		DC_set_error(desc, DESC_BAD_PARAMETER_NUMBER_ERROR,
 				"bad parameter number");
 		return SQL_ERROR;
 	}
 	if (RecNumber > opts->allocated)
 	{
-inolog("APDSetField RecN=%d allocated=%d\n", RecNumber, opts->allocated);
+inolog("%s RecN=%d allocated=%d\n", func, RecNumber, opts->allocated);
 		parameter_bindings_set(opts, RecNumber, TRUE);
 		/* DC_set_error(desc, DESC_BAD_PARAMETER_NUMBER_ERROR,
 				"bad parameter number");
@@ -698,7 +694,6 @@ inolog("APDSetField RecN=%d allocated=%d\n", RecNumber, opts->allocated);
 	switch (FieldIdentifier)
 	{
 		case SQL_DESC_TYPE:
-			reset_a_parameter_binding(opts, RecNumber);
 			opts->parameters[para_idx].CType = CAST_PTR(SQLSMALLINT, Value);
 			break;
 		case SQL_DESC_DATETIME_INTERVAL_CODE:
@@ -727,19 +722,18 @@ inolog("APDSetField RecN=%d allocated=%d\n", RecNumber, opts->allocated);
 			opts->parameters[para_idx].CType = CAST_PTR(SQLSMALLINT, Value);
 			break;
 		case SQL_DESC_DATA_PTR:
+			unbind = FALSE;
 			opts->parameters[para_idx].buffer = Value;
 			break;
 		case SQL_DESC_INDICATOR_PTR:
-			if (Value != opts->parameters[para_idx].used)
-			{
-				ret = SQL_ERROR;
-				DC_set_error(desc, DESC_INVALID_DESCRIPTOR_IDENTIFIER, "INDICATOR != OCTET_LENGTH_PTR"); 
-			}
+			unbind = FALSE;
+			opts->parameters[para_idx].indicator = Value;
 			break;
 		case SQL_DESC_OCTET_LENGTH:
 			opts->parameters[para_idx].buflen = CAST_PTR(Int4, Value);
 			break;
 		case SQL_DESC_OCTET_LENGTH_PTR:
+			unbind = FALSE;
 			opts->parameters[para_idx].used = Value;
 			break;
 		case SQL_DESC_PRECISION:
@@ -756,6 +750,9 @@ inolog("APDSetField RecN=%d allocated=%d\n", RecNumber, opts->allocated);
 			DC_set_error(desc, DESC_INVALID_DESCRIPTOR_IDENTIFIER,
 				"invaid descriptor identifier"); 
 	}
+	if (unbind)
+		opts->parameters[para_idx].buffer = NULL;
+
 	return ret;
 }
 
@@ -860,8 +857,11 @@ inolog("IPDSetField RecN=%d allocated=%d\n", RecNumber, ipdopts->allocated);
 	switch (FieldIdentifier)
 	{
 		case SQL_DESC_TYPE:
-			reset_a_iparameter_binding(ipdopts, RecNumber);
-			ipdopts->parameters[para_idx].SQLType = CAST_PTR(SQLSMALLINT, Value);
+			if (ipdopts->parameters[para_idx].SQLType != CAST_PTR(SQLSMALLINT, Value))
+			{
+				reset_a_iparameter_binding(ipdopts, RecNumber);
+				ipdopts->parameters[para_idx].SQLType = CAST_PTR(SQLSMALLINT, Value);
+			}
 			break;
 		case SQL_DESC_DATETIME_INTERVAL_CODE:
 			switch (ipdopts->parameters[para_idx].SQLType)
@@ -957,7 +957,7 @@ ARDGetField(DescriptorClass *desc, SQLSMALLINT RecNumber,
 				break;
 			case SQL_DESC_INDICATOR_PTR:
 				rettype = SQL_IS_POINTER;
-				ptr = bookmark ? bookmark->used : NULL;
+				ptr = bookmark ? bookmark->indicator : NULL;
 				break;
 			case SQL_DESC_OCTET_LENGTH_PTR:
 				rettype = SQL_IS_POINTER;
@@ -1046,7 +1046,7 @@ ARDGetField(DescriptorClass *desc, SQLSMALLINT RecNumber,
 			break;
 		case SQL_DESC_INDICATOR_PTR:
 			rettype = SQL_IS_POINTER;
-			ptr = opts->bindings[row_idx].used;
+			ptr = opts->bindings[row_idx].indicator;
 			break;
 		case SQL_DESC_OCTET_LENGTH_PTR:
 			rettype = SQL_IS_POINTER;
@@ -1194,7 +1194,7 @@ inolog("APDGetField RecN=%d allocated=%d\n", RecNumber, opts->allocated);
 			break;
 		case SQL_DESC_INDICATOR_PTR:
 			rettype = SQL_IS_POINTER;
-			ptr = opts->parameters[para_idx].used;
+			ptr = opts->parameters[para_idx].indicator;
 			break;
 		case SQL_DESC_OCTET_LENGTH:
 			ival = opts->parameters[para_idx].buflen;
@@ -1370,7 +1370,7 @@ IPDGetField(DescriptorClass *desc, SQLSMALLINT RecNumber,
 		SQLINTEGER *StringLength)
 {
 	RETCODE		ret = SQL_SUCCESS;
-	SQLINTEGER	ival = 0, len, rettype = 0;
+	SQLINTEGER	ival = 0, len = 0, rettype = 0;
 	PTR		ptr = NULL;
 	const IPDFields	*ipdopts = (const IPDFields *) (desc + 1);
 	SQLSMALLINT	para_idx;
@@ -1649,7 +1649,7 @@ PGAPI_SetConnectAttr(HDBC ConnectionHandle,
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 			mylog("SQL_ATTR_ENLIST_IN_DTC %p request received\n", Value);
 			if (conn->connInfo.xa_opt != 0)	
-				return EnlistInDtc(conn, Value, conn->connInfo.xa_opt);
+				return CALL_EnlistInDtc(conn, Value, conn->connInfo.xa_opt);
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
 #endif /* WIN32 */
 			unsupported = TRUE;
@@ -1737,7 +1737,7 @@ PGAPI_SetDescField(SQLHDESC DescriptorHandle,
 	RETCODE		ret = SQL_SUCCESS;
 	DescriptorClass *desc = (DescriptorClass *) DescriptorHandle;
 
-	mylog("%s h=%p rec=%d field=%d val=%p,%d\n", func, DescriptorHandle, RecNumber, FieldIdentifier, Value, BufferLength);
+	mylog("%s h=%p(%d) rec=%d field=%d val=%p,%d\n", func, DescriptorHandle, desc->desc_type, RecNumber, FieldIdentifier, Value, BufferLength);
 	switch (desc->desc_type)
 	{
 		case SQL_ATTR_APP_ROW_DESC:
@@ -1787,7 +1787,7 @@ PGAPI_SetStmtAttr(HSTMT StatementHandle,
 	CSTR func = "PGAPI_SetStmtAttr";
 	StatementClass *stmt = (StatementClass *) StatementHandle;
 
-	mylog("%s Handle=%p %d,%u\n", func, StatementHandle, Attribute, Value);
+	mylog("%s Handle=%p %d,%u(%p)\n", func, StatementHandle, Attribute, Value, Value);
 	switch (Attribute)
 	{
 		case SQL_ATTR_ENABLE_AUTO_IPD:	/* 15 */
